<?php
/**
 * Main class for importing demo content.
 *
 * @author FitWP
 * @link http://fitwp.com
 */

/**
 * Main class for importing demo content.
 */
class FitWP_Demo_Importer {
	/**
	 * The base dir for demo files.
	 * @var string
	 */
	protected $dir;

	/**
	 * Process importing request
	 * Redirect when success to avoid user refresh browser
	 * @return bool
	 */
	public function import() {
		if ( ! $this->validate() ) {
			return false;
		}

		// Start importing
		set_time_limit( 0 );
		$this->process();

		return true;
	}

	/**
	 * Select parts to be imported.
	 * Can be extended in each theme.
	 */
	public function process() {
		$this->import_content();
		$this->import_theme_options();
		$this->import_widgets();

		$this->import_menus();
		$this->import_sliders();

		$this->set_pages();
	}

	/**
	 * Import posts, pages, menus, custom post types.
	 */
	protected function import_content() {
		$file = $this->dir . 'content.xml';
		if ( ! file_exists( $file ) ) {
			return;
		}

		if ( ! defined( 'WP_LOAD_IMPORTERS' ) ) {
			define( 'WP_LOAD_IMPORTERS', true );
		}
		require_once FITWP_DEMO_DIR . 'lib/wordpress-importer/wordpress-importer.php';
		require_once FITWP_DEMO_DIR . 'inc/content-importer.php';
		ob_start();
		$importer                    = new FitWP_Content_Importer();
		$importer->fetch_attachments = true;
		$importer->image_dir         = $this->dir . 'images';
		$importer->import( $file );
		ob_end_clean();
	}

	/**
	 * Import theme options.
	 */
	protected function import_theme_options() {
		$file = $this->dir . 'theme-options.dat';
		if ( ! file_exists( $file ) ) {
			return;
		}

		$file_data = file_get_contents( $file );
		$data      = @unserialize( $file_data );

		// Import images.
		$data['mods'] = $this->import_customizer_images( $data['mods'] );

		// Import custom options.
		foreach ( $data['mods'] as $key => $val ) {
			set_theme_mod( $key, $val );
		}
	}

	/**
	 * Imports images for settings saved as mods.
	 *
	 * @param array $mods An array of customizer mods.
	 *
	 * @return array The mods array with any new import data.
	 */
	protected function import_customizer_images( $mods ) {
		foreach ( $mods as $key => $val ) {
			if ( ! $this->is_image_url( $val ) ) {
				continue;
			}
			$data = $this->sideload_image( $val );
			if ( is_wp_error( $data ) ) {
				continue;
			}

			$mods[ $key ] = $data->url;

			// Handle header image controls.
			if ( isset( $mods[ $key . '_data' ] ) ) {
				$mods[ $key . '_data' ] = $data;
				update_post_meta( $data->attachment_id, '_wp_attachment_is_custom_header', get_stylesheet() );
			}
		}

		return $mods;
	}

	/**
	 * Checks to see whether a string is an image url or not.
	 *
	 * @param string $string The string to check.
	 *
	 * @return bool Whether the string is an image url or not.
	 */
	protected function is_image_url( $string = '' ) {
		return is_string( $string ) && preg_match( '/\.(jpe?g|png|gif)/i', $string );
	}

	/**
	 * Taken from the core media_sideload_image function and modified to return an array of data instead of html.
	 *
	 * @param string $file The image file path.
	 *
	 * @return array An array of image data.
	 */
	protected function sideload_image( $file ) {
		$data = new stdClass();

		if ( ! function_exists( 'media_handle_sideload' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/media.php' );
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
			require_once( ABSPATH . 'wp-admin/includes/image.php' );
		}
		if ( ! empty( $file ) ) {
			// Set variables for storage, fix file filename for query strings.
			preg_match( '/[^\?]+\.(jpe?g|jpe|gif|png)\b/i', $file, $matches );
			$file_array         = array();
			$file_array['name'] = basename( $matches[0] );

			// Download file to temp location.
			$file_array['tmp_name'] = download_url( $file );

			// If error storing temporarily, return the error.
			if ( is_wp_error( $file_array['tmp_name'] ) ) {
				return $file_array['tmp_name'];
			}

			// Do the validation and storage stuff.
			$id = media_handle_sideload( $file_array, 0 );

			// If error storing permanently, unlink.
			if ( is_wp_error( $id ) ) {
				@unlink( $file_array['tmp_name'] );

				return $id;
			}

			// Build the object to return.
			$meta                = wp_get_attachment_metadata( $id );
			$data->attachment_id = $id;
			$data->url           = wp_get_attachment_url( $id );
			$data->thumbnail_url = wp_get_attachment_thumb_url( $id );
			$data->height        = $meta['height'];
			$data->width         = $meta['width'];
		}

		return $data;
	}

	/**
	 * Import widgets.
	 */
	protected function import_widgets() {
		$file = $this->dir . 'widgets.wie';
		if ( ! file_exists( $file ) ) {
			return;
		}
		require_once FITWP_DEMO_DIR . 'lib/widgets-importer.php';

		$data     = json_decode( file_get_contents( $file ) );
		$importer = new FitWP_Widgets_Importer();
		$importer->import( $data );
	}

	/**
	 * Import menu locations. Menu items are already imported in import_content() function.
	 */
	protected function import_menus() {
		$file = $this->dir . 'menus.json';
		if ( ! file_exists( $file ) ) {
			return;
		}

		$data  = json_decode( file_get_contents( $file ), true );
		$menus = wp_get_nav_menus();

		foreach ( $data as $key => $val ) {
			foreach ( $menus as $menu ) {
				if ( $val && $menu->slug == $val ) {
					$data[ $key ] = absint( $menu->term_id );
				}
			}
		}

		set_theme_mod( 'nav_menu_locations', $data );
	}

	/**
	 * Import revolution sliders.
	 */
	protected function import_sliders() {
		if ( ! class_exists( 'RevSlider' ) ) {
			return;
		}

		$files = glob( $this->dir . 'sliders/*.zip' );
		if ( empty( $files ) ) {
			return;
		}

		ob_start();
		$slider = new RevSlider();
		foreach ( $files as $file ) {
			$slider->importSliderFromPost( true, true, $file );
		}
		ob_clean();
	}

	/**
	 * Set frontpage and blog page
	 */
	protected function set_pages() {
		$file = $this->dir . 'pages.json';
		if ( ! file_exists( $file ) ) {
			return;
		}

		$data = json_decode( file_get_contents( $file ), true );

		if ( isset( $data['home_page'] ) ) {
			$home = get_page_by_title( $data['home_page'] );

			if ( $home ) {
				update_option( 'show_on_front', 'page' );
				update_option( 'page_on_front', $home->ID ); // Front Page
			}
		}

		if ( isset( $data['blog_page'] ) ) {
			$blog = get_page_by_title( $data['blog_page'] );

			if ( $blog ) {
				update_option( 'page_for_posts', $blog->ID ); // Front Page
			}
		}
	}

	/**
	 * Validate form request.
	 * If success, then get the demo name.
	 * @return bool
	 */
	protected function validate() {
		$demo  = (string) filter_input( INPUT_POST, 'demo' );
		$nonce = (string) filter_input( INPUT_POST, 'nonce' );
		if ( ! wp_verify_nonce( $nonce, "import-$demo" ) ) {
			return false;
		}
		$this->dir = FITWP_DEMO_DIR . "demos/$demo/";

		return true;
	}
}
