<?php
/**
 * Admin page class for adding Demo Import page in the admin area.
 *
 * @package FitWP Demo
 * @author FitWP
 * @link http://fitwp.com
 */

/**
 * Admin page class.
 */
class FitWP_Demo_Page {
	/**
	 * The importer object that run the import process.
	 * @var object
	 */
	protected $importer;

	/**
	 * Constructor.
	 *
	 * @param object $importer The importer object that run the import process.
	 */
	public function __construct( $importer ) {
		$this->importer = $importer;
	}

	/**
	 * Initialize.
	 */
	public function init() {
		add_action( 'admin_menu', array( $this, 'menu' ) );
	}

	/**
	 * Add new menu under Tools.
	 */
	public function menu() {
		$page = add_management_page(
			__( 'Import Demo', 'fitwp-demo' ),
			__( 'Import Demo', 'fitwp-demo' ),
			'edit_theme_options',
			'import-demo',
			array( $this, 'page' )
		);
		add_action( "admin_print_styles-$page", array( $this, 'enqueue' ) );
		add_action( "load-$page", array( $this, 'add_notice' ) );
		add_action( "load-$page", array( $this, 'import' ) );
	}

	/**
	 * Enqueue scripts and styles.
	 */
	public function enqueue() {
		wp_enqueue_style( 'fitwp-demo', FITWP_DEMO_URL . 'css/import.css' );
	}

	/**
	 * Show import demo page content.
	 */
	public function page() {
		?>
		<div class="wrap">
			<h1><?php esc_html_e( get_admin_page_title() ); ?></h1>
			<h3><?php esc_html_e( 'Please select a demo to import', 'fitwp-demo' ); ?></h3>

			<?php
			$demos = $this->get_demos();
			foreach ( $demos as $demo ) {
				?>
				<form action="" method="post" class="media">
					<?php wp_nonce_field( "import-$demo", 'nonce' ); ?>
					<input type="hidden" name="demo" value="<?php esc_attr_e( $demo ); ?>">
					<div class="image">
						<img src="<?php echo esc_url( FITWP_DEMO_URL . "demos/$demo/preview.jpg" ); ?>">
					</div>
					<h4><?php esc_html_e( sprintf( __( 'Demo %s', 'fitwp-demo' ), $demo ) ); ?></h4>
					<?php submit_button( __( 'Import', 'fitwp-demo' ) ); ?>
				</form>
				<?php
			}
			?>

		</div>
		<?php
	}

	/**
	 * Hook to show import messages.
	 */
	public function add_notice() {
		add_action( 'admin_notices', array( $this, 'notice' ) );
	}

	/**
	 * Show import messages.
	 */
	public function notice() {
		settings_errors( 'fitwp-demo' );
	}

	/**
	 * Import demo.
	 */
	public function import() {
		if ( ! filter_input( INPUT_POST, 'demo' ) ) {
			return;
		}
		if ( $this->importer->import() ) {
			add_settings_error( 'fitwp-demo', 'success', __( 'Demo imported. Have fun!', 'fitwp-demo' ), 'updated' );
		} else {
			add_settings_error( 'fitwp-demo', 'error', __( 'Something wrong with the import process. Please try again later!', 'fitwp-demo' ), 'error' );
		}
	}

	/**
	 * Get list of demos.
	 */
	protected function get_demos() {
		$folders = glob( FITWP_DEMO_DIR . 'demos/*', GLOB_ONLYDIR );

		return array_map( 'basename', $folders );
	}
}
